@@ -4,9 +4,11 @@ import copy
 import enum
 import math
 import types
+import builtins
 import warnings
 import dataclasses
-from typing import Iterable, Optional, Sequence, Type, Union
+from typing import Optional, Union
+from collections.abc import Iterator, Sequence
 
 import bpack.utils
 import bpack.typing
@@ -31,6 +33,7 @@ __all__ = [
     "BinFieldDescriptor",
     "get_field_descriptor",
     "set_field_descriptor",
+    "flat_fields_iterator",
     "BASEUNITS_ATTR_NAME",
     "BYTEORDER_ATTR_NAME",
     "BITORDER_ATTR_NAME",
@@ -61,8 +64,12 @@ def _resolve_type(type_):
     """
     if bpack.utils.is_sequence_type(type_):
         etype = bpack.utils.effective_type(type_)
-        rtype = copy.copy(type_)
-        rtype.__args__ = (etype,)
+        try:
+            # this is for typing.List and typing.Sequence
+            rtype = type_.copy_with(etype)
+        except AttributeError:
+            # if the `list[T]` syntax is used then `copy_with` is not available
+            rtype = type_.__class_getitem__(etype)
     elif bpack.typing.is_annotated(type_):
         rtype = bpack.utils.effective_type(type_)
     else:
@@ -74,10 +81,13 @@ def _resolve_type(type_):
 class BinFieldDescriptor:
     """Descriptor for bpack fields.
 
-    See also :func:`bpack.filed` for a description of the attributes.
+    .. seealso::
+
+        :func:`bpack.descriptors.field` for a description of the attributes.
+
     """
 
-    type: Optional[Type] = None  # noqa: A003
+    type: Optional[builtins.type] = None  # noqa: A003
     size: Optional[int] = None  #: item size
     offset: Optional[int] = None
     signed: Optional[bool] = None
@@ -86,7 +96,7 @@ class BinFieldDescriptor:
 
     def _validate_type(self):
         if self.type is None:
-            raise TypeError(f"invalid type '{self.type}'")
+            raise TypeError(f"invalid type '{self.type!r}'")
 
     def _validate_size(self):
         msg = f"invalid size: {self.size!r} (must be a positive integer)"
@@ -146,21 +156,21 @@ class BinFieldDescriptor:
             if not self.is_int_type():
                 warnings.warn(
                     f"the 'signed' parameter will be ignored for non-integer "
-                    f"type: '{self.type}'"
+                    f"type: '{self.type!r}'"
                 )
         if self.repeat is not None:
             self._validate_repeat()
             if not self.is_sequence_type() and self.repeat is not None:
                 raise TypeError(
                     f"repeat parameter specified for non-sequence type: "
-                    f"{self.type}"
+                    f"{self.type!r}"
                 )
         if bpack.utils.is_enum_type(self.type):
             self._validate_enum_type()
         elif self.is_sequence_type() and self.repeat is None:
             raise TypeError(
                 f"no 'repeat' parameter specified for sequence type "
-                f"{self.type}"
+                f"{self.type!r}"
             )
 
     def is_int_type(self) -> bool:
@@ -187,7 +197,7 @@ class BinFieldDescriptor:
             return False
         return True
 
-    def update_from_type(self, type_: Type):
+    def update_from_type(self, type_: type):
         """Update the field descriptor according to the specified type."""
         if self.type is not None:
             raise TypeError("the type attribute is already set")
@@ -235,10 +245,10 @@ def field(
     initialized to describe the field of a binary record.
 
     :param size: int
-        size of the field in :class:`EBaseUnits`
+        size of the field in :class:`bpack.enums.EBaseUnits`
     :param offset: int
         offset of the field w.r.t. the beginning of the record
-        (expressed in :class:`EBaseUnits`)
+        (expressed in :class:`bpack.enums.EBaseUnits`)
     :param signed: bool
         True if an `int` field is signed, False otherwise.
         This parameter must not be specified for non `int` fields.
@@ -246,7 +256,7 @@ def field(
         length of the sequence for `sequence` fields, i.e. fields
         consisting in multiple items having the same data type.
         This parameter must not be specified if the data type is not a
-        sequence type (e.g. `List`).
+        sequence type (e.g. :class:`typing.List`).
     :param metadata:
         additional metadata to be attached the the field descriptor.
     :param kwargs:
@@ -266,7 +276,7 @@ def field(
 
 def is_field(obj) -> bool:
     """Return true if an ``obj`` can be considered is a field descriptor."""
-    return (
+    return bool(
         isinstance(obj, Field)
         and obj.metadata
         and METADATA_KEY in obj.metadata
@@ -312,7 +322,7 @@ def set_field_descriptor(
     if type_ != _resolve_type(field_type):
         raise TypeError(
             f"type mismatch between BinFieldDescriptor.type ({type_!r}) and "
-            f"filed.type ({field.type!r})"
+            f"field.type ({field.type!r})"
         )
     new_metadata = {
         METADATA_KEY: types.MappingProxyType(field_descr_metadata),
@@ -344,7 +354,7 @@ def _get_default_size(type_, baseunits: EBaseUnits) -> Union[int, None]:
     # if bpack.utils.is_enum_type(type_):
     #     if bpack.utils.is_int_type(type_):
     #         signbit = 1 if any(item.value < 0 for item in type_) else 0
-    #         bits = signbit + max(item.value.bit_lenght() for item in type_)
+    #         bits = signbit + max(item.value.bit_length() for item in type_)
     #         if baseunits is EBaseUnits.BITS:
     #             if bits <= 8:
     #                 return 1
@@ -385,7 +395,7 @@ def _get_effective_byteorder(
 
 
 @classdecorator
-def descriptor(
+def descriptor(  # noqa: CCR001
     cls,
     *,
     size: Optional[int] = None,
@@ -490,7 +500,7 @@ def descriptor(
             field_descr.offset = auto_offset
         elif field_descr.offset < auto_offset:
             raise DescriptorConsistencyError(
-                f"invalid offset for filed n. {idx}: {field_}"
+                f"invalid offset for field n. {idx}: {field_}"
             )
 
         set_field_descriptor(field_, field_descr)
@@ -611,7 +621,7 @@ def baseunits(obj) -> EBaseUnits:
 
 
 def byteorder(obj) -> EByteOrder:
-    """Return the byte order of a binary record descriptor (endianess)."""
+    """Return the byte order of a binary record descriptor (endianness)."""
     try:
         return getattr(obj, BYTEORDER_ATTR_NAME)
     except AttributeError:
@@ -628,11 +638,12 @@ def bitorder(obj) -> Union[EBitOrder, None]:
 
 def field_descriptors(
     descriptor, pad: bool = False
-) -> Iterable[BinFieldDescriptor]:
-    """Return the list of field descriptors for the input record descriptor.
+) -> Iterator[BinFieldDescriptor]:
+    """Iterate the input record descriptor and return binary field descriptors.
 
-    Items are instances of the :class:`BinFieldDescriptor` class describing
-    characteristics of each field of the input binary record descriptor.
+    Returned items are instances of the :class:`BinFieldDescriptor` class
+    describing characteristics of each field of the input binary record
+    descriptor.
 
     If the ``pad`` parameter is set to True then also generate dummy field
     descriptors for padding elements necessary to take into account offsets
@@ -659,3 +670,37 @@ def field_descriptors(
     else:
         for field_ in fields(descriptor):
             yield get_field_descriptor(field_)
+
+
+def flat_fields_iterator(descriptor, offset: int = 0) -> Iterator[Field]:
+    """Recursively iterate on fields of a record descriptor.
+
+    The behavior of this function is similar to the one of
+    :func:`bpack.descriptors.fields` if the input descriptor do not
+    contain fields that are descriptors (nested).
+    The main difference is that this one is an iterator while
+    :func:`bpack.descriptors.fields` returns a tuple.
+
+    If the input descriptor is nested (i.e. has fields that are
+    descriptors), then a the it is visited recursively to return all
+    the fields belonging to the main descriptor and to the nested ones.
+
+    The nested descriptors are replaced by their fields and the
+    returned sequence of fields is *flat*.
+
+    .. note::
+
+        please note that in case of nested descriptors, the returned fields
+        are copy of the original ones, with the `offset` attribute adjusted
+        to the relative to the beginning of the root descriptor.
+    """
+    for field_ in bpack.fields(descriptor):
+        fd = get_field_descriptor(field_)
+        fd.offset += offset
+
+        if bpack.is_descriptor(field_.type):
+            yield from flat_fields_iterator(field_.type, offset=fd.offset)
+        else:
+            field_ = copy.copy(field_)
+            set_field_descriptor(field_, fd)
+            yield field_
